import argparse
from getpass import getpass
import re

import itchiodl


def main():
    parser = argparse.ArgumentParser(prog="python -m hstp", description="Build an ")

    parser.add_argument(
        "-k", "--api-key", help="Use API key instead of username/password"
    )

    parser.add_argument(
        "-p",
        "--platform",
        help="Platform to download for (default: all), will accept values like 'windows', 'linux', 'osx' and android",
    )

    parser.add_argument(
        "-j",
        "--jobs",
        type=int,
        default=4,
        help="Number of concurrent downloads, defaults to 4",
    )

    parser.add_argument(
        "--download-publisher",
        type=str,
        help="Download all games from a specific publisher",
    )

    parser.add_argument(
        "--download-game",
        type=str,
        help="Download a specific game, should be in the format publisher.itch.io/game",
    )

    args = parser.parse_args()

    l = ""

    if not args.api_key:
        user = input("Username: ")
        password = getpass("Password: ")
        l = itchiodl.LoginAPI(user, password)
    else:
        l = args.api_key

    lib = itchiodl.Library(l, args.jobs)

    if args.download_publisher:
        lib.load_games(args.download_publisher)
    elif args.download_game:
        matches = re.match(r"https://(.+)\.itch\.io/(.+)", args.download_game)
        lib.load_game(matches.group(1), matches.group(2))
    else:
        lib.load_owned_games()

    lib.download_library(args.platform)


if __name__ == "__main__":
    main()



import json
from bs4 import BeautifulSoup as soup
import requests


warning = (
    "Will print the response text (Please be careful as "
    + "this may contain personal data or allow others to login to your account):"
)


def LoginWeb(user, password):
    """Login to itch.io using webscraping"""
    session = requests.Session()

    # GET the page first so we have a valid CSRF token value
    login1 = session.get("https://itch.io/login")
    s = soup(login1.text, "html.parser")
    csrf_token = s.find("input", {"name": "csrf_token"})["value"]

    # Now POST the login
    r = session.post(
        "https://itch.io/login",
        {"username": user, "password": password, "csrf_token": csrf_token},
    )

    if r.status_code != 200:
        raise RuntimeError

    return session


def LoginAPI(user, password):
    """Login to itch.io using API"""
    r = requests.post(
        "https://api.itch.io/login",
        {"username": user, "password": password, "source": "desktop"},
    )
    if r.status_code != 200:
        print(f"Error: {r.status_code} is not 200")
        print(warning)
        print(r.text)
        raise RuntimeError
    t = json.loads(r.text)

    if not t["success"]:
        print("Error: success key is not true")
        print(warning)
        print(r.text)
        raise RuntimeError

    return t["key"]["key"]



from getpass import getpass
import itchiodl


def main():
    user = input("Username: ")
    password = getpass("Password: ")

    l = itchiodl.LoginWeb(user, password)

    url = input("Bundle URL: ")
    b = itchiodl.Bundle(l, url)
    b.load_games()


if __name__ == "__main__":
    main()



import re
import json
import urllib
import datetime
from pathlib import Path
import requests

from itchiodl import utils


class Game:
    """Representation of a game download"""

    def __init__(self, data):
        self.data = data["game"]
        self.name = self.data["title"]
        self.publisher = self.data["user"]["username"]
        self.link = self.data["url"]
        if "game_id" in data:
            self.id = data["id"]
            self.game_id = data["game_id"]
        else:
            self.id = False
            self.game_id = self.data["id"]

        matches = re.match(r"https://(.+)\.itch\.io/(.+)", self.link)
        self.game_slug = matches.group(2)
        self.publisher_slug = matches.group(1)

        self.files = []
        self.downloads = []
        self.dir = (
            Path(".")
            / utils.clean_path(self.publisher_slug)
            / utils.clean_path(self.game_slug)
        )

    def load_downloads(self, token):
        """Load all downloads for this game"""
        self.downloads = []
        if self.id:
            r = requests.get(
                f"https://api.itch.io/games/{self.game_id}/uploads?download_key_id={self.id}",
                headers={"Authorization": token},
            )
        else:
            r = requests.get(
                f"https://api.itch.io/games/{self.game_id}/uploads",
                headers={"Authorization": token},
            )
        j = r.json()
        for d in j["uploads"]:
            self.downloads.append(d)

    def download(self, token, platform):
        """Download a singular file"""
        print("Downloading", self.name)

        # if out_folder.with_suffix(".json").exists():
        #    print(f"Skipping Game {self.name}")
        #    return

        self.load_downloads(token)

        self.dir.mkdir(parents=True, exist_ok=True)

        for d in self.downloads:
            if (
                platform is not None
                and d["traits"]
                and f"p_{platform}" not in d["traits"]
            ):
                print(f"Skipping {self.name} for platform {d['traits']}")
                continue
            self.do_download(d, token)

        with self.dir.with_suffix(".json").open("w") as f:
            json.dump(
                {
                    "name": self.name,
                    "publisher": self.publisher,
                    "link": self.link,
                    "itch_id": self.id,
                    "game_id": self.game_id,
                    "itch_data": self.data,
                },
                f,
                indent=2,
            )

    def do_download(self, d, token):
        """Download a single file, checking for existing files"""
        print(f"Downloading {d['filename']}")

        filename = d["filename"] or d["display_name"] or d["id"]

        out_file = self.dir / filename

        if out_file.exists():
            print(f"File Already Exists! {filename}")
            md5_file = out_file.with_suffix(".md5")
            if md5_file.exists():
                with md5_file.open("r") as f:
                    md5 = f.read().strip()
                    if md5 == d["md5_hash"]:
                        print(f"Skipping {self.name} - {filename}")
                        return
                    print(f"MD5 Mismatch! {filename}")
            else:
                md5 = utils.md5sum(str(out_file))
                if md5 == d["md5_hash"]:
                    print(f"Skipping {self.name} - {filename}")

                    # Create checksum file
                    with md5_file.open("w") as f:
                        f.write(d["md5_hash"])
                    return
                # Old Download or corrupted file?
                corrupted = False
                if corrupted:
                    out_file.remove()
                    return

            old_dir = self.dir / "old"
            old_dir.mkdir(exist_ok=True)

            print(f"Moving {filename} to old/")
            timestamp = datetime.datetime.now().strftime("%Y-%m-%d")
            out_file.rename(old_dir / f"{timestamp}-{filename}")

        # Get UUID
        r = requests.post(
            f"https://api.itch.io/games/{self.game_id}/download-sessions",
            headers={"Authorization": token},
        )
        j = r.json()

        # Download
        if self.id:
            url = (
                f"https://api.itch.io/uploads/{d['id']}/"
                + f"download?api_key={token}&download_key_id={self.id}&uuid={j['uuid']}"
            )
        else:
            url = (
                f"https://api.itch.io/uploads/{d['id']}/"
                + f"download?api_key={token}&uuid={j['uuid']}"
            )
        # response_code = urllib.request.urlopen(url).getcode()
        try:
            utils.download(url, self.dir, self.name, filename)
        except utils.NoDownloadError:
            print("Http response is not a download, skipping")

            with open("errors.txt", "a") as f:
                f.write(
                    f""" Cannot download game/asset: {self.game_slug}
                    Publisher Name: {self.publisher_slug}
                    Path: {out_file}
                    File: {filename}
                    Request URL: {url}
                    This request failed due to a missing response header
                    This game/asset has been skipped please download manually
                    ---------------------------------------------------------\n """
                )

            return
        except urllib.error.HTTPError as e:
            print("This one has broken due to an HTTP error!!")

            with open("errors.txt", "a") as f:
                f.write(
                    f""" Cannot download game/asset: {self.game_slug}
                    Publisher Name: {self.publisher_slug}
                    Path: {out_file}
                    File: {filename}
                    Request URL: {url}
                    Request Response Code: {e.code}
                    Error Reason: {e.reason}
                    This game/asset has been skipped please download manually
                    ---------------------------------------------------------\n """
                )

            return

        # Verify
        if utils.md5sum(out_file) != d["md5_hash"]:
            print(f"Failed to verify {filename}")
            return

        # Create checksum file
        with out_file.with_suffix(".md5").open("w") as f:
            f.write(d["md5_hash"])



from bs4 import BeautifulSoup as soup


class Bundle:
    """Bundle class, represents a bundle on itch.io"""

    def __init__(self, login, url):
        self.login = login
        self.url = url

    def load_games(self):
        """Load all games in the bundle via web scraping"""
        i = 1

        r = self.login.get(self.url)
        s = soup(r.text, "html.parser")
        pages = int(s.select("span.pager_label a")[-1].text)
        while i < pages:
            if self.load_game(i):
                i += 1
                print(f"Processing Page {i} of {pages}")

    def load_game(self, i):
        """Load 1 game. This will refresh the game afterwards, as the csrf token
        will update"""
        r = self.login.get(f"{self.url}?page={i}")
        s = soup(r.text, "html.parser")
        for g in s.select("div.game_row"):
            name = g.select("h2 a")[0].text
            if f := g.find("form"):
                print(f"Processing {name}")

                game_id = f.find("input", {"name": "game_id"})["value"]
                csrf_token = f.find("input", {"name": "csrf_token"})["value"]

                data = {"action": "claim", "game_id": game_id, "csrf_token": csrf_token}

                r = self.login.post(f"{self.url}?page={i}", data=data)
                return False
            # else:
            #    print(f"Skipping {name} - Already in Library")
        return True



import json
from concurrent.futures import ThreadPoolExecutor
import functools
import threading
import requests
from bs4 import BeautifulSoup

from itchiodl.game import Game
from itchiodl.utils import NoDownloadError


class Library:
    """Representation of a user's game library"""

    def __init__(self, login, jobs=4):
        self.login = login
        self.games = []
        self.jobs = jobs

    def load_game_page(self, page):
        """Load a page of games via the API"""
        print("Loading page", page)
        r = requests.get(
            f"https://api.itch.io/profile/owned-keys?page={page}",
            headers={"Authorization": self.login},
        )
        j = json.loads(r.text)

        for s in j["owned_keys"]:
            self.games.append(Game(s))

        return len(j["owned_keys"])

    def load_owned_games(self):
        """Load all games in the library via the API"""
        page = 1
        while True:
            n = self.load_game_page(page)
            if n == 0:
                break
            page += 1

    def load_game(self, publisher, title):
        """Load a game by publisher and title"""
        rsp = requests.get(
            f"https://{publisher}.itch.io/{title}/data.json",
            headers={"Authorization": self.login},
        )
        j = json.loads(rsp.text)
        game_id = j["id"]
        gsp = requests.get(
            f"https://api.itch.io/games/{game_id}/uploads",
            headers={"Authorization": self.login},
        )
        k = gsp.json()
        if k != {"uploads": {}}:
            self.games.append(Game(k))
            return
        print(f"{title} is a purchased game.")
        i = 1
        while self.games == []:
            j = self.load_game_page(i)

            self.games = [
                x
                for x in self.games
                if x.link == f"https://{publisher}.itch.io/{title}"
            ]

            if j == 0:
                break
            i += 1

        if self.games == []:
            print(f"Cannot find {title} in owned keys, you may not own it.")

    def load_games(self, publisher):
        """Load all games by publisher"""
        rsp = requests.get(f"https://{publisher}.itch.io")
        soup = BeautifulSoup(rsp.text, "html.parser")
        for link in soup.select("a.game_link"):
            game_id = link.get("data-label").split(":")[1]
            gsp = requests.get(
                f"https://api.itch.io/games/{game_id}",
                headers={"Authorization": self.login},
            )
            k = json.loads(gsp.text)
            self.games.append(Game(k))

    def download_library(self, platform=None):
        """Download all games in the library"""
        with ThreadPoolExecutor(max_workers=self.jobs) as executor:
            i = [0, 0]
            l = len(self.games)
            lock = threading.RLock()

            def dl(i, g):
                try:
                    g.download(self.login, platform)
                    with lock:
                        i[0] += 1
                    print(f"Downloaded {g.name} ({i[0]} of {l})")
                except NoDownloadError as e:
                    print(e)
                    i[1] += 1

            r = executor.map(functools.partial(dl, i), self.games)
            for _ in r:
                pass
            print(f"Downloaded {i[0]} Games, {i[1]} Errors")



import re
import sys
import hashlib
import requests


class NoDownloadError(Exception):
    """No download found exception"""


def download(url, path, name, file):
    """Downloads a file from a url and saves it to a path, skips it if it already exists."""

    desc = f"{name} - {file}"
    print(f"Downloading {desc}")
    rsp = requests.get(url, stream=True)

    if (
        rsp.headers.get("content-length") is None
        or rsp.headers.get("Content-Disposition") is None
    ):
        raise NoDownloadError("Http response is not a download, skipping")

    cd = rsp.headers.get("Content-Disposition")

    filename_re = re.search(r'filename="(.+)"', cd)
    if filename_re is None:
        filename = file
    else:
        filename = filename_re.group(1)

    with open(f"{path}/{filename}", "wb") as f:
        for chunk in rsp.iter_content(10240):
            f.write(chunk)

    print(f"Downloaded {filename}")
    return f"{path}/{filename}", True


def clean_path(path):
    """Cleans a path on windows"""
    if sys.platform in ["win32", "cygwin", "msys"]:
        path_clean = re.sub(r"[<>:|?*\"\/\\]", "-", path)
        return path_clean
    return path


def md5sum(path):
    """Returns the md5sum of a file"""
    md5 = hashlib.md5()
    with path.open("rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            md5.update(chunk)
    return md5.hexdigest()
